#include <amxmodx>
#include <nvault>
#include <sqlx>
#include <ze_core>

// Constants.
new const g_szVaultName[] = "ZE_EscapeInfects"
new const g_szLogFile[] = "SQL_EscapeInfects.log"
new const g_szTable[] = "\
CREATE TABLE IF NOT EXISTS `ze_escapes_infects` ( \
`AuthID` varchar(64) NOT NULL, \
`Escapes` int(16) NOT NULL DEFAULT 0, \
`Infects` int(16) NOT NULL DEFAULT 0, \
PRIMARY KEY (AuthID)); \
"

// Variables.
new g_iVault,
	g_iSaveType

// Arrays.
new g_iEscapes[MAX_PLAYERS+1],
	g_iInfects[MAX_PLAYERS+1]

// String.
new g_szAuthData[MAX_PLAYERS+1][MAX_AUTHID_LENGTH]

// MySQL Handle.
new Handle:g_hTuple

public plugin_natives()
{
	register_native("ze_get_user_escapes", "__native_get_user_escapes")
	register_native("ze_set_user_escapes", "__native_set_user_escapes")
	register_native("ze_get_user_infects", "__native_get_user_infects")
	register_native("ze_set_user_infects", "__native_set_user_infects")
}

public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] Addon: Escapes & Infects", "1.0", "z0h1r-LK")

	// Cvars.
	bind_pcvar_num(register_cvar("ze_esc_inf_savetype", "0"), g_iSaveType)

	// Commands.
	register_clcmd("say /escapes", "cmd_PrintEscapes")
	register_clcmd("say /infects", "cmd_PrintInfects")
	register_clcmd("say_team /escapes", "cmd_PrintEscapes")
	register_clcmd("say_team /infects", "cmd_PrintInfects")
}

public plugin_cfg()
{
	switch (g_iSaveType)
	{
		case 0: // nVault.
		{
			if ((g_iVault = nvault_open(g_szVaultName)) == INVALID_HANDLE)
				set_fail_state("Error in opening the Vault (-1)")
		}
		case 1: // MySQL.
		{
			MySQL_Init()
		}
	}
}

public plugin_end()
{
	switch (g_iSaveType)
	{
		case 0:
		{
			if (g_iVault != INVALID_HANDLE)
				nvault_close(g_iVault)
		}
		case 1:
		{
			if (g_hTuple != Empty_Handle)
				SQL_FreeHandle(g_hTuple)
		}
	}
}

public MySQL_Init()
{
	new szHost[64], szUser[64], szPass[64], szDB[64]

	// sql.cfg
	get_pcvar_string(register_cvar("amx_sql_host", "127.0.0.1", FCVAR_PROTECTED), szHost, charsmax(szHost))
	get_pcvar_string(register_cvar("amx_sql_user", "root", FCVAR_PROTECTED), szUser, charsmax(szUser))
	get_pcvar_string(register_cvar("amx_sql_pass", "", FCVAR_PROTECTED), szPass, charsmax(szPass))
	get_pcvar_string(register_cvar("amx_sql_db", "amx", FCVAR_PROTECTED), szDB, charsmax(szDB))
	new const iTimeOut = get_pcvar_num(register_cvar("amx_sql_timeout", "60", FCVAR_PROTECTED))

	g_hTuple = SQL_MakeDbTuple(szHost, szUser, szPass, szDB, iTimeOut)

	new Handle:hSQLConnection, szError[256], iError

	// Connect to SQL database.
	hSQLConnection = SQL_Connect(g_hTuple, iError, szError, charsmax(szError))

	if (hSQLConnection != Empty_Handle)
	{
		log_amx("[MySQL][Escp/Infs] Successfully connected to host: %s (ALL IS OK).", szHost)

		// Frees SQL handle.
		SQL_FreeHandle(hSQLConnection)
	}
	else
	{
		// Disable plugin.
		set_fail_state("[MySQL][Escp/Infs] Failed to connect to MySQL database: %s.", szError)
	}

	// Create Table.
	SQL_ThreadQuery(g_hTuple, "query_CreateTable", g_szTable)
}

public query_CreateTable(iFailState, Handle:hQuery, szError[], iError, szData[], iSize, Float:flQueueTime)
{
	SQL_IsFail(iFailState, iError, szError, g_szLogFile)
}

public cmd_PrintEscapes(id)
{
	ze_colored_print(id, "%L", LANG_PLAYER, "MSG_ESCAPES", g_iEscapes[id])
	return PLUGIN_CONTINUE
}

public cmd_PrintInfects(id)
{
	ze_colored_print(id, "%L", LANG_PLAYER, "MSG_INFECTS", g_iInfects[id])
	return PLUGIN_CONTINUE
}

public client_authorized(id, const szAuthID[])
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	copy(g_szAuthData[id], charsmax(g_szAuthData[]), szAuthID)
	read_Data(id)
}

public client_disconnected(id, bool:drop, message[], maxlen)
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	write_Data(id)

	// Clear arrays.
	g_iEscapes[id] = 0
	g_iInfects[id] = 0
	g_szAuthData[id] = NULL_STRING
}

public ze_user_infected(iVictim, iInfector)
{
	if (!iInfector)
		return

	g_iInfects[iInfector]++
}

public ze_roundend(iWinTeam)
{
	if (iWinTeam == ZE_TEAM_HUMAN)
	{
		new iPlayers[MAX_PLAYERS], iAliveNum
		get_players(iPlayers, iAliveNum, "ah")

		for (new id, i = 0; i < iAliveNum; i++)
		{
			id = iPlayers[i]

			if (ze_is_user_zombie(id))
				continue

			g_iEscapes[id]++
		}
	}
}

/**
 * -=| Functions |=-
 */
public read_Data(id)
{
	switch (g_iSaveType)
	{
		case 0:
		{
			new szData[33]
			if (nvault_get(g_iVault, g_szAuthData[id], szData, charsmax(szData)))
			{
				new szEscapes[16], szInfects[16]
				parse(szData, szEscapes, charsmax(szEscapes), szInfects, charsmax(szInfects))
				g_iEscapes[id] = str_to_num(szEscapes)
				g_iInfects[id] = str_to_num(szInfects)
			}
			else
			{
				g_iEscapes[id] = 0
				g_iInfects[id] = 0
			}
		}
		case 1:
		{
			new szQuery[128], szData[4]
			num_to_str(id, szData, charsmax(szData))
			formatex(szQuery, charsmax(szQuery), "SELECT * FROM `ze_escapes_infects` WHERE `AuthID` = '%s';", g_szAuthData[id])
			SQL_ThreadQuery(g_hTuple, "query_SelectData", szQuery, szData, charsmax(szData))
		}
	}
}

public query_SelectData(iFailState, Handle:hQuery, szError[], iError, szData[], iSize, Float:flQueueTime)
{
	if (SQL_IsFail(iFailState, iError, szError, g_szLogFile))
		return

	new const id = str_to_num(szData)

	if (!SQL_NumResults(hQuery))
	{
		g_iEscapes[id] = 0
		g_iInfects[id] = 0
	}
	else
	{
		g_iEscapes[id] = SQL_ReadResult(hQuery, 1)
		g_iInfects[id] = SQL_ReadResult(hQuery, 2)
	}
}

public write_Data(id)
{
	switch (g_iSaveType)
	{
		case 0:
		{
			new szData[33]
			formatex(szData, charsmax(szData), "%d %d", g_iEscapes[id], g_iInfects[id])
			nvault_pset(g_iVault, g_szAuthData[id], szData)
		}
		case 1:
		{
			new szQuery[128]
			formatex(szQuery, charsmax(szQuery), "REPLACE INTO `ze_escapes_infects` (`AuthID`, `Escapes`, `Infects`) VALUES ('%s', %i, %i);", g_szAuthData[id], g_iEscapes[id], g_iInfects[id])
			SQL_ThreadQuery(g_hTuple, "query_SetData", szQuery)
		}
	}
}

public query_SetData(iFailState, Handle:hQuery, szError[], iError, szData[], iSize, Float:flQueueTime)
{
	SQL_IsFail(iFailState, iError, szError, g_szLogFile)
}

/**
 * -=| Natives |=-
 */
public __native_get_user_escapes(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return NULLENT
	}

	return g_iEscapes[id]
}

public __native_get_user_infects(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return NULLENT
	}

	return g_iInfects[id]
}

public __native_set_user_escapes(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return
	}

	g_iEscapes[id] = get_param(2)
	write_Data(id)
}

public __native_set_user_infects(plugin_id, num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return
	}

	g_iInfects[id] = get_param(2)
	write_Data(id)
}